@@ -10,6 +10,7 @@ import numpy as np
 
 import openmdao.api as om
 from openmdao.core.driver import Driver
+from openmdao.utils.units import convert_units
 from openmdao.utils.assert_utils import assert_near_equal, assert_warning
 from openmdao.utils.general_utils import printoptions
 from openmdao.utils.testing_utils import use_tempdirs
@@ -638,6 +639,30 @@ class TestDriver(unittest.TestCase):
         con = case.get_constraints()
         assert_near_equal(con['comp1.y1'][0], ((38.0 * 5 / 9) + 77.0) * 3.5, 1e-8)
 
+    def test_units_compute_totals(self):
+        p = om.Problem()
+
+        p.model.add_subsystem('stuff', om.ExecComp(['y = x', 'cy = x'],
+                                                   x={'units': 'inch'},
+                                                   y={'units': 'kg'},
+                                                   cy={'units': 'kg'}),
+                              promotes=['*'])
+
+        p.model.add_design_var('x', units='ft')
+        p.model.add_objective('y', units='lbm')
+        p.model.add_constraint('cy', units='lbm', lower=0)
+
+        p.setup()
+
+        p['x'] = 1.0
+        p.run_model()
+
+        J_driver = p.driver._compute_totals()
+
+        fact = convert_units(1.0, 'kg/inch', 'lbm/ft')
+        assert_near_equal(J_driver['stuff.y', 'x'][0,0], fact, 1e-5)
+        assert_near_equal(J_driver['stuff.cy', 'x'][0,0], fact, 1e-5)
+
     def test_units_error_messages(self):
         prob = om.Problem()
         model = prob.model
@@ -812,7 +837,7 @@ class TestDriverMPI(unittest.TestCase):
 
     N_PROCS = 2
 
-    def test_unsupported_distributed(self):
+    def test_distrib_desvar_unsupported(self):
 
         class MyDriver(Driver):
 
@@ -836,9 +861,139 @@ class TestDriverMPI(unittest.TestCase):
         with self.assertRaises(RuntimeError) as context:
             prob.final_setup()
 
-        msg = "Distributed design variables are not supported by this driver, but the following variables are distributed: [p.invec]"
-        self.assertEqual(str(context.exception), msg)
+        self.assertEqual(str(context.exception),
+                         "Distributed design variables are not supported by this driver, "
+                         "but the following variables are distributed: [p.invec]")
 
+    def test_distrib_desvar_get_set(self):
+
+        comm = MPI.COMM_WORLD
+        size = 3 if comm.rank == 0 else 2
+
+        class DistribComp(om.ExplicitComponent):
+
+            def setup(self):
+                self.options['distributed'] = True
+
+                self.add_input('w', val=1.) # this will connect to a non-distributed IVC
+                self.add_input('x', shape=size) # this will connect to a distributed IVC
+
+                self.add_output('y', shape=1) # all-gathered output, duplicated on all procs
+                self.add_output('z', shape=size) # distributed output
+
+                self.declare_partials('y', 'x')
+                self.declare_partials('y', 'w')
+                self.declare_partials('z', 'x')
+
+            def compute(self, inputs, outputs):
+                x = inputs['x']
+                local_y = np.sum((x-5)**2)
+                y_g = np.zeros(self.comm.size)
+                self.comm.Allgather(local_y, y_g)
+
+                outputs['y'] = np.sum(y_g) + (inputs['w']-10)**2
+                outputs['z'] = x**2
+
+            def compute_partials(self, inputs, J):
+                x = inputs['x']
+                J['y', 'x'] = 2*(x-5)
+                J['y', 'w'] = 2*(inputs['w']-10)
+                J['z', 'x'] = np.diag(2*x)
+
+        p = om.Problem()
+        model = p.model
+        driver = p.driver
+
+        # distributed indep var, 'x'
+        d_ivc = model.add_subsystem('d_ivc', om.IndepVarComp(distributed=True),
+                                    promotes=['*'])
+        d_ivc.add_output('x', 2*np.ones(size))
+
+        # non-distributed indep var, 'w'
+        ivc = model.add_subsystem('ivc', om.IndepVarComp(distributed=False),
+                                  promotes=['*'])
+        ivc.add_output('w', size)
+
+        # distributed component, 'dc'
+        model.add_subsystem('dc', DistribComp(), promotes=['*'])
+
+        # distributed design var, 'x'
+        model.add_design_var('x', lower=-100, upper=100)
+        model.add_objective('y')
+
+        # driver that supports distributed design vars
+        driver.supports._read_only = False
+        driver.supports['distributed_design_vars'] = True
+
+        # run model
+        p.setup()
+        p.run_model()
+
+        # get distributed design var
+        driver.get_design_var_values(get_remote=None)
+
+        assert_near_equal(driver.get_design_var_values(get_remote=True)['d_ivc.x'],
+                          [2, 2, 2, 2, 2])
+
+        assert_near_equal(driver.get_design_var_values(get_remote=False)['d_ivc.x'],
+                          2*np.ones(size))
+
+        # set distributed design var, set_remote=True
+        driver.set_design_var('d_ivc.x', [3, 3, 3, 3, 3], set_remote=True)
+
+        assert_near_equal(driver.get_design_var_values(get_remote=True)['d_ivc.x'],
+                          [3, 3, 3, 3, 3])
+
+        # set distributed design var, set_remote=False
+        if comm.rank == 0:
+            driver.set_design_var('d_ivc.x', 5.0*np.ones(size), set_remote=False)
+        else:
+            driver.set_design_var('d_ivc.x', 9.0*np.ones(size), set_remote=False)
+
+        assert_near_equal(driver.get_design_var_values(get_remote=True)['d_ivc.x'],
+                          [5, 5, 5, 9, 9])
+
+        # run driver
+        p.run_driver()
+
+        assert_near_equal(p.get_val('dc.y', get_remote=True), [81, 96])
+        assert_near_equal(p.get_val('dc.z', get_remote=True), [25, 25, 25, 81, 81])
+
+    def test_distrib_desvar_bug(self):
+        class MiniModel(om.Group):
+            def setup(self):
+                self.add_subsystem('dv', om.IndepVarComp('x', 3.0))
+                self.add_subsystem('comp', om.ExecComp('y = (x-2)**2'))
+                self.connect('dv.x', 'comp.x')
+
+                self.add_design_var('dv.x', lower=-10, upper=10)
+
+        class ParModel(om.ParallelGroup):
+            def setup(self):
+                self.add_subsystem('g0', MiniModel())
+                self.add_subsystem('g1', MiniModel())
+
+        p = om.Problem()
+
+        pg = p.model.add_subsystem('par_group', ParModel())
+        p.model.add_subsystem('obj', om.ExecComp('f = y0 + y1'))
+
+        p.model.connect('par_group.g0.comp.y', 'obj.y0')
+        p.model.connect('par_group.g1.comp.y', 'obj.y1')
+
+        p.model.add_objective('obj.f')
+
+        p.driver = om.ScipyOptimizeDriver()
+
+        p.setup()
+        p.run_driver()
+
+        p.model.list_outputs()
+
+        dvs = p.driver.get_design_var_values(get_remote=True)
+
+        assert_near_equal(dvs['par_group.g0.dv.x'], 2)
+        assert_near_equal(dvs['par_group.g1.dv.x'], 2)
 
 if __name__ == "__main__":
     unittest.main()
